<?php
if (!isset($SITE)){
	header ("Location: ../index.php");	// back to index/startpage if someone tries an
	exit;  								//  page to load without menu system//
}
$pageName		= 'incMeteoalarm.php';
$pageVersion	= '1.01k 2012-10-28';
$SITE['wsModules'][$pageName] = 'version: ' . $pageVersion;
$pageFile = basename(__FILE__);			// check to see this is the real script
if ($pageFile <> $pageName) {
	$SITE['wsModules'][$pageFile]	= 'this file loaded instead of '.$pageName;
}
echo '<!-- module '.$pageFile.' ==== '.$SITE['wsModules'][$pageFile]." -->".PHP_EOL;
#--------------------------------------------------------------------------------------------------
# retrieve weather alarm information from meteoalarm.eu  
# and return array with retrieved data in the desired language
#--------------------------------------------------------------------------------------------------
class meteoAlarm {
	# public variables
	public $area		= 'BE004';  	// test GR016   BE004
	public $langAlarm	= 'en_UK';		// only to retreive data, translated by langtransstr
	# private variables
	private $enableCache= true;			// cache should be anabled when frequent request are made. Keep in mind that the data is only refreshed every hour
	private $cachePath	= '../cache/';	// cache dir is created when not available
	private $cacheTime 	= 1800; 		// Cache expiration time Default: 300 seconds = advised by meteoalarm.eu
	private $iconUrl 	= 'http://www.meteoalarm.eu/';  //  Location of the weather icons 
	private $cache_file	= '';
	private $timeOffset	= 0;			// times on some warnpages differ from each other with this offset
	private $timeformat = 'd.m.Y h:i';	// 28.07.2012 05:00 CET 
	private $timeFormat = 'd.m.Y h:i';	// 28.07.2012 05:00 CET 
	private $timeOnlyFormat = 'H:i';
// $SITE['timeOnlyFormat'] = 'H:i'; 
	private $dataOK		= false;
	private $urlPart 	= array(
		'http://www.meteoalarm.eu/',
		'handheld.php?level=2&area=',
		'$area',
		'&day=',
		'1',
		'&lang=',
		'$langAlarm',
		'&mod1=0&mod2=0'
	);
	private $fullUrl 	= '';
// 							URL should become = 'http://www.meteoalarm.eu/handheld.php?level=2&area=BE001&day=1&lang=en_UK&mod1=0&mod2=0';
	private $rawData	= '';
	private $charset	= 'ISO-8859-1'; // output charset, input is utf8
	private $warnColors = array(
		00 => '#99ff99',   // green .. No particular awareness of the weather is required.
		10 => '#FFFF66',   // yellow ..The weather is potentially dangerous.
		20 => '#FECC33',   // orange ..The weather is dangerous.
		30 => '#FF6666'    // red ..   The weather is very dangerous. Exceptionally intense meteorological phenomena have been forecast.
	);
#--------------------------------------------------------------------------------------------------
# public functions	
#--------------------------------------------------------------------------------------------------
	public function getMeteoAlarm($area = '') {
		global $SITE, $pageName;
		#
		# check environment setting
		if ( isset ($SITE['warnOffset']) && ($SITE['warnOffset'] > -3601) && ($SITE['warnOffset'] < 7200) ) { $this->timeOffset = $SITE['warnOffset']; } 
		if ( isset ($SITE['timeFormat']) ) 		{ $this->timeFormat 	= $SITE['timeFormat']; }
		if ( isset ($SITE['timeOnlyFormat']) )	{ $this->timeOnlyFormat = $SITE['timeOnlyFormat']; }
		#
		# clean user input
		#
		$area = trim($area);  // user input by call of this function
		if (strlen($area) >= 2) {$this->area = $area;}
		$this->langAlarm ='en_UK';
		if (isset ($SITE['cacheDir']) && !empty($SITE['cacheDir']) ) {
			$this->cachePath = $SITE['cacheDir'];
		} else {
			$this->enableCache = false;
		}
		#--------------------------------------------------------------------------------------------------
		# check if data (for this location) is in cache
		if ($this->enableCache && !empty($this->cachePath)){
			$this->cache_file = $this->cachePath .$pageName.$this->area.'_'.$this->langAlarm;
			$returnArray=$this->loadFromCache();  	// load from cache returns data only when its data is valid
			if (!empty($returnArray)) {				// if data is in cache and valid return data to calling program
				return $returnArray;
			}  // eo return to calling program
		}  // eo check cache
		#----------------------------------------------------------------------------------------------
		# combine user constants and input (1)location (2)units for temp etc  to required url
		#----------------------------------------------------------------------------------------------
		$this->urlPart[6]	= $this->langAlarm;
		$this->urlPart[4]	= '0';     // 0 = today   1 = tomorrow
		$this->urlPart[2]	= $this->area;
		$this->fullUrl		= '';
		for ($i = 0; $i < count($this->urlPart); $i++){
			$this->fullUrl .= $this->urlPart[$i];
		}
		#----------------------------------------------------------------------------------------------
		if ($this->makeRequest()){				
			$strStart	= '<tr bgcolor=';	// start of first warning
			$strNoWarn	= '"#99ff99"';		// no warning when this color
			$lenNoWarn	= strlen($strNoWarn);
			$strFrom	= '<b>valid from</b>';
			$strUntil	= '<b>Until</b>';
			$strEndBr	= '<br>';
			$strTypes	= '<b>awareness types: </b>';
			$strLevel	= '<b>Awareness Level: </b>';
			$strWarn	= 'warn_';
			$strMsg		= '<td colspan="4" bgcolor="#ffffff">';
			$strEndMsg	= '&nbsp;';
			$returnArray = array();
			$pos = strpos($this->rawData, $strStart,0)+strlen($strStart);
			$color = substr($this->rawData,$pos,$lenNoWarn);
			if ($color == $strNoWarn) {	// no data this time
				$returnArray[0]['msg'] = 'no warnings';
			} else 
			{
				$moreData = true;
				$cntWarn = 0;
				$replace	= array ("\n", "\t");
				while ($moreData) {	
					$returnArray[$cntWarn]['color']	= $color;
					$pos 	= strpos($this->rawData,$strFrom,$pos);
					$start	= $pos + strlen ($strFrom);
					$pos	= strpos($this->rawData,$strUntil,$start);
					$text	= trim(substr($this->rawData,$start,$pos - $start));
					echo '<!-- from = '.$text;
					$unixFrom	= strtotime ($text)	+ $this->timeOffset;
					$returnArray[$cntWarn]['from']	= date ($this->timeFormat, $unixFrom );
					$start = $pos + strlen ($strUntil);
					$pos	= strpos($this->rawData,$strEndBr,$start);
					$text	= trim(substr($this->rawData,$start,$pos - $start));
					echo ' until = '.$text. ' -->'.PHP_EOL;
					$unixUntil	= strtotime ($text)	+ $this->timeOffset;						
					if (date ('Ymd' , $unixFrom ) <> date ('Ymd' , $unixUntil) ) {
						$returnArray[$cntWarn]['until']	= date ($this->timeFormat, $unixUntil );
					} else {
						$returnArray[$cntWarn]['until']	= date ($this->timeOnlyFormat, $unixUntil );
					}
					echo '<!-- from = '.$returnArray[$cntWarn]['from'].' - until = '.$returnArray[$cntWarn]['until'].' -->'.PHP_EOL;
					$pos = strpos($this->rawData,$strTypes,$pos);
					$start = $pos + strlen ($strTypes);
					$pos = strpos($this->rawData,$strEndBr,$start);
					$returnArray[$cntWarn]['types']	= trim(substr($this->rawData,$start,$pos - $start));
					$pos = strpos($this->rawData,$strLevel,$pos);
					$start = $pos + strlen ($strLevel);
					$pos = strpos($this->rawData,'</td>',$start);
					$text = substr($this->rawData,$start,$pos - $start);
					$returnArray[$cntWarn]['level']	= trim(str_replace($replace, '', $text));
					$pos = strpos($this->rawData,$strWarn,$pos);
					$start = $pos + strlen ($strWarn);
					$pos = strpos($this->rawData,'"',$start);
					$returnArray[$cntWarn]['img']	= substr($this->rawData,$start,$pos - $start);
					$pos = strpos($this->rawData,$strMsg,$pos);
					$start = $pos + strlen ($strMsg);
					$pos = strpos($this->rawData,$strEndMsg,$start);
					$text = substr($this->rawData,$start,$pos - $start);
					$text = trim(str_replace($replace, '', $text));
					$returnArray[$cntWarn]['msg']	= utf8_decode($text);
					$pos = strpos($this->rawData, $strStart,$pos);
					if (!$pos) { $moreData=false;  // no more data
						} else {
							$pos = $pos + strlen($strStart);
							$color = substr($this->rawData,$pos,$lenNoWarn);
							$cntWarn=$cntWarn+1;
						}
				} // end while
			}
			$this->writeToCache($returnArray);
			$this->rawData  = '';
			return $returnArray;
		} 
		else 
		{		
		return false;
		}
	}  // eof getMeteoAlarm
//
	private function loadFromCache(){
		if (file_exists($this->cache_file)){	
			$file_time = filectime($this->cache_file);
			$now = time();
			$diff = ($now-$file_time);		
			if ($diff <= $this->cacheTime){
				echo "<!-- Severe weatherdata ($this->cache_file) loaded from cache -->".PHP_EOL;
				$returnArray =  unserialize(file_get_contents($this->cache_file));
				return $returnArray;
			}
		}	
	} // eof loadFromCache
	
	private function writeToCache($data){
		if (!$this->enableCache){echo "<!-- no cache -->".PHP_EOL; return;}
		if (!file_exists($this->cachePath)){
			mkdir($this->cachePath, 0777);   // attempt to make the cache dir
		}
	//			print_r ($data); return;
		if (!file_put_contents($this->cache_file, serialize($data))){   
			echo PHP_EOL."<!-- Could not save data to cache $this->cache_file. Please make sure your cache directory exists and is writable. -->".PHP_EOL;
		} else {echo "<!-- Severe weatherdata ($this->cache_file) saved to cache  -->".PHP_EOL;}
	} // eof writeToCache
	
	private function makeRequest(){	
		global $SITE;
		$test= false;
		if ($test) {
			$testFile = 'cache/test1.html';
			echo "<!-- TESTING   Severe weatherdata loaded from test file: $testFile  -->".PHP_EOL;
			$this->rawData  = file_get_contents($testFile);
		} else {
			echo "<!-- Severe weatherdata loaded from $this->fullUrl  -->".PHP_EOL;
			$ch = curl_init();
			curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt ($ch, CURLOPT_URL, $this->fullUrl);
			curl_setopt($ch, CURLOPT_USERAGENT, $_SERVER['HTTP_USER_AGENT']);
			curl_setopt ($ch, CURLOPT_TIMEOUT, 10);
			curl_setopt($ch, CURLOPT_FOLLOWLOCATION, $SITE['curlFollow']);
			$this->rawData = curl_exec ($ch);
			curl_close ($ch);
		}
		if (empty($this->rawData)){
			echo "<!-- ERROR Severe weatherdata empty,($this->fullUrl)  could not be loaded  -->".PHP_EOL;
			$this->dataOK = false;
			return false;
		}
		$dataError = strpos($this->rawData, 'Service Unavailable' ,0);	
		$dataDetect= strpos($this->rawData, 'InformationToHelpDiagnose:' ,0);
		if($dataError > 0  || $dataDetect > 0) {   // error string found
			echo "<!-- ERROR Severe weatherdata ($this->fullUrl) Service Unavailable  -->".PHP_EOL;
			$this->dataOK = false;
			return false;
		}
		$dataOK = strpos($this->rawData, $this->urlPart[1] ,0);	
		if(!$dataOK) {   // no good data found
			echo "<!-- ERROR Severe weatherdata ($this->fullUrl) Service returns no good data -->".PHP_EOL;
			$this->dataOK = false;
			return false;
		}		
		$this->dataOK = true;
		return true;	
	} // eof makeRequest
} // eo class
?>
